import random

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomFloatGenerator(RandomGenerator):
    """
    Class to generate random float number
    """

    def generate(self, *args, **kwargs) -> float:
        min_limit = defaults.NUMBER_MIN_LIMIT
        max_limit = defaults.NUMBER_MAX_LIMIT
        decimals = defaults.DECIMALS
        random_number = random.uniform(min_limit, max_limit)
        rounded_number = round(random_number, decimals)
        return rounded_number



from dataclasses import is_dataclass

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators import random_data_class_generator
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomListGenerator(RandomGenerator):
    """
    Class to generate random list
    """

    def generate(self, *args, **kwargs) -> list:
        max_length = defaults.MAX_LIST_LENGTH

        default_value_type = defaults.DEFAULT_VALUE_TYPE

        value_type = kwargs.get("value_type", default_value_type)

        if is_dataclass(value_type):
            generator = random_data_class_generator.RandomDataClassGenerator()
        else:
            generator = defaults.TYPING_GENERATORS[value_type]()

        random_list = []
        for _ in range(max_length):
            if is_dataclass(value_type):
                list_value = generator.generate(value_type)
            else:
                list_value = generator.generate()

            random_list.append(list_value)

        return random_list



from pathlib import Path

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator
from dataclass_bakery.generators.random_str_generator import RandomStrGenerator


class RandomPathGenerator(RandomGenerator):
    def generate(self, *args, **kwargs) -> Path:
        generator = RandomStrGenerator()
        folders = [generator.generate() for _ in range(defaults.MAX_PATH_LENGTH)]
        random_path = "/".join(folders)
        return random_path



from dataclasses import is_dataclass
from typing import _GenericAlias, Any, Literal, Union

from dataclass_bakery.generators.defaults import TYPING_GENERATORS


class RandomDataClassGenerator:
    def generate(self, data_class: Any, *args, **kwargs) -> Any:
        random_data = {}
        for field_name, field_type in data_class.__annotations__.items():

            arguments = kwargs.get(field_name, {})

            if isinstance(field_type, _GenericAlias) and field_type.__origin__ == Union:
                has_new_type = False
                for argument in field_type.__args__:
                    if argument != type(None):
                        has_new_type = True
                        field_type = argument

                    if (
                        not hasattr(field_type, "__origin__")
                        or field_type.__origin__ != Union
                    ):
                        break

                if not has_new_type:
                    raise TypeError(f"Union without Typing in dataclass {field_name}")

            if (
                isinstance(field_type, _GenericAlias)
                and field_type.__origin__ != Literal
            ):
                arguments_lenght = len(field_type.__args__)
                if arguments_lenght > 1:  # Is a Dict
                    arguments["key_type"] = field_type.__args__[0]
                    arguments["value_type"] = field_type.__args__[1]
                elif arguments_lenght > 0:  # Is a List or Tuple
                    arguments["value_type"] = field_type.__args__[0]

                field_type = field_type.__origin__

            elif (
                isinstance(field_type, _GenericAlias)
                and field_type.__origin__ == Literal
            ):
                arguments["options"] = field_type.__args__
                field_type = field_type.__origin__

            if is_dataclass(field_type):
                generator = RandomDataClassGenerator()
                field_randomized = generator.generate(field_type, **arguments)
            else:
                generator_class = TYPING_GENERATORS.get(field_type)
                generator = generator_class()
                field_randomized = generator.generate(**arguments)

            random_data[field_name] = field_randomized

        return data_class(**random_data)



import random

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomIntGenerator(RandomGenerator):
    """
    Class to generate random int number
    """

    def generate(self, *args, **kwargs) -> int:
        min_limit = defaults.NUMBER_MIN_LIMIT
        max_limit = defaults.NUMBER_MAX_LIMIT
        random_number = random.randint(min_limit, max_limit)
        return random_number



from distutils.core import setup
from setuptools import find_packages


with open("README.rst", "r") as readme:
    README_TEXT = readme.read()

setup(
    name="dataclass-bakery",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    version="0.0.5",
    description="Dataclass Bakery offers you a smart way to create fixtures for testing in Python with dataclasses",
    include_package_data=True,
    author="Miguel Jiménez",
    author_email="miguelflg13@gmx.com",
    url="https://github.com/miguelFLG13/dataclass-bakery",
    download_url="https://github.com/miguelFLG13/dataclass-bakery/tarball/0.0.5",
    long_description=README_TEXT,
    keywords="testing dataclass bakery fixture",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    python_requires=">=3.6",
)
